@@ -47,27 +47,27 @@ static cmd_status_t cmd_ping_execute(char *args, char **message);
 static cmd_status_t cmd_aclk_state(char *args, char **message);
 static cmd_status_t cmd_version(char *args, char **message);
 static cmd_status_t cmd_dumpconfig(char *args, char **message);
-#ifdef ENABLE_ACLK
-static cmd_status_t cmd_remove_node(char *args, char **message);
-#endif
+static cmd_status_t cmd_remove_stale_node(char *args, char **message);
+static cmd_status_t cmd_mark_stale_nodes_ephemeral(char *args, char **message);
 
 static command_info_t command_info_array[] = {
-        {"help", cmd_help_execute, CMD_TYPE_HIGH_PRIORITY},                  // show help menu
-        {"reload-health", cmd_reload_health_execute, CMD_TYPE_ORTHOGONAL},   // reload health configuration
-        {"reopen-logs", cmd_reopen_logs_execute, CMD_TYPE_ORTHOGONAL},       // Close and reopen log files
-        {"shutdown-agent", cmd_exit_execute, CMD_TYPE_EXCLUSIVE},            // exit cleanly
-        {"fatal-agent", cmd_fatal_execute, CMD_TYPE_HIGH_PRIORITY},          // exit with fatal error
-        {"reload-claiming-state", cmd_reload_claiming_state_execute, CMD_TYPE_ORTHOGONAL}, // reload claiming state
-        {"reload-labels", cmd_reload_labels_execute, CMD_TYPE_ORTHOGONAL},   // reload the labels
-        {"read-config", cmd_read_config_execute, CMD_TYPE_CONCURRENT},
-        {"write-config", cmd_write_config_execute, CMD_TYPE_ORTHOGONAL},
-        {"ping", cmd_ping_execute, CMD_TYPE_ORTHOGONAL},
-        {"aclk-state", cmd_aclk_state, CMD_TYPE_ORTHOGONAL},
-        {"version", cmd_version, CMD_TYPE_ORTHOGONAL},
-        {"dumpconfig", cmd_dumpconfig, CMD_TYPE_ORTHOGONAL},
-#ifdef ENABLE_ACLK
-        {"remove-stale-node", cmd_remove_node, CMD_TYPE_ORTHOGONAL}
-#endif
+    {"help", "", "Show this help menu.", cmd_help_execute, CMD_TYPE_HIGH_PRIORITY},                // show help menu
+    {"reload-health", "", "Reload health configuration.", cmd_reload_health_execute, CMD_TYPE_ORTHOGONAL}, // reload health configuration
+    {"reopen-logs", "", "Close and reopen log files.", cmd_reopen_logs_execute, CMD_TYPE_ORTHOGONAL},     // Close and reopen log files
+    {"shutdown-agent", "", "Cleanup and exit the netdata agent.", cmd_exit_execute, CMD_TYPE_EXCLUSIVE},          // exit cleanly
+    {"fatal-agent", "", "Log the state and halt the netdata agent.", cmd_fatal_execute, CMD_TYPE_HIGH_PRIORITY},        // exit with fatal error
+    {"reload-claiming-state", "", "Reload agent claiming state from disk.", cmd_reload_claiming_state_execute, CMD_TYPE_ORTHOGONAL}, // reload claiming state
+    {"reload-labels", "", "Reload all localhost labels.", cmd_reload_labels_execute, CMD_TYPE_ORTHOGONAL},                 // reload the labels
+    {"read-config", "", "", cmd_read_config_execute, CMD_TYPE_CONCURRENT},
+    {"write-config", "", "", cmd_write_config_execute, CMD_TYPE_ORTHOGONAL},
+    {"ping", "", "Return with 'pong' if agent is alive.", cmd_ping_execute, CMD_TYPE_ORTHOGONAL},
+    {"aclk-state", "[json]",  "Returns current state of ACLK and Netdata Cloud connection. (optionally in json).", cmd_aclk_state, CMD_TYPE_ORTHOGONAL},
+    {"version", "", "Returns the netdata version.", cmd_version, CMD_TYPE_ORTHOGONAL},
+    {"dumpconfig", "", "Returns the current netdata.conf on stdout.", cmd_dumpconfig, CMD_TYPE_ORTHOGONAL},
+    {"remove-stale-node", "<node_id | machine_guid | hostname | ALL_NODES>",
+     "Marks one or all disconnected nodes as ephemeral, and removes them\n      so that they are no longer available for queries, from both this\n      Netdata Agent dashboard and Netdata Cloud.", cmd_remove_stale_node, CMD_TYPE_ORTHOGONAL},
+    {"mark-stale-nodes-ephemeral", "<node_id | machine_guid | hostname | ALL_NODES>",
+     "Marks one or all disconnected nodes as ephemeral, while keeping their retention\n      available for queries on both this Netdata Agent dashboard and Netdata Cloud", cmd_mark_stale_nodes_ephemeral, CMD_TYPE_ORTHOGONAL},
 };
 
 /* Mutexes for commands of type CMD_TYPE_ORTHOGONAL */
@@ -109,39 +109,26 @@ static command_lock_t *cmd_unlock_by_type[] = {
 static cmd_status_t cmd_help_execute(char *args, char **message)
 {
     (void)args;
+    CLEAN_BUFFER *wb = buffer_create(0, NULL);
 
-    *message = mallocz(MAX_COMMAND_LENGTH);
-    strncpyz(*message,
-             "\nThe commands are (arguments are in brackets):\n"
-             "help\n"
-             "    Show this help menu.\n"
-             "reload-health\n"
-             "    Reload health configuration.\n"
-             "reload-labels\n"
-             "    Reload all labels.\n"
-             "save-database\n"
-             "    Save internal DB to disk for memory mode save.\n"
-             "reopen-logs\n"
-             "    Close and reopen log files.\n"
-             "shutdown-agent\n"
-             "    Cleanup and exit the netdata agent.\n"
-             "fatal-agent\n"
-             "    Log the state and halt the netdata agent.\n"
-             "reload-claiming-state\n"
-             "    Reload agent claiming state from disk.\n"
-             "ping\n"
-             "    Return with 'pong' if agent is alive.\n"
-             "aclk-state [json]\n"
-             "    Returns current state of ACLK and Cloud connection. (optionally in json).\n"
-             "dumpconfig\n"
-             "    Returns the current netdata.conf on stdout.\n"
-#ifdef ENABLE_ACLK
-             "remove-stale-node node_id|machine_guid|hostname|ALL_NODES\n"
-             "    Unregisters and removes a node from the cloud.\n"
-#endif
-             "version\n"
-             "    Returns the netdata version.\n",
-             MAX_COMMAND_LENGTH - 1);
+    buffer_strcat(wb, "The commands are:\n\n");
+    for(size_t i = 0; i < _countof(command_info_array); i++) {
+        const command_info_t *t = &command_info_array[i];
+        if(!t->help || !t->help[0]) continue;
+
+        buffer_strcat(wb, "  ");
+        buffer_strcat(wb, t->cmd_str);
+        if(t->params && t->params[0]) {
+            buffer_putc(wb, ' ');
+            buffer_strcat(wb, t->params);
+        }
+        buffer_putc(wb, '\n');
+        buffer_strcat(wb, "      ");
+        buffer_strcat(wb, t->help);
+        buffer_strcat(wb, "\n\n");
+    }
+
+    *message = strdupz(buffer_tostring(wb));
     return CMD_STATUS_SUCCESS;
 }
 
@@ -177,9 +164,7 @@ static cmd_status_t cmd_exit_execute(char *args, char **message)
 
     nd_log_limits_unlimited();
     netdata_log_info("COMMAND: Cleaning up to exit.");
-    netdata_cleanup_and_exit(0, NULL, NULL, NULL);
-    exit(0);
-
+    netdata_exit_gracefully(EXIT_REASON_CMD_EXIT, true);
     return CMD_STATUS_SUCCESS;
 }
 
@@ -193,17 +178,42 @@ static cmd_status_t cmd_fatal_execute(char *args, char **message)
     return CMD_STATUS_SUCCESS;
 }
 
-static cmd_status_t cmd_reload_claiming_state_execute(char *args, char **message)
-{
-    (void)args;
-    (void)message;
-#if defined(DISABLE_CLOUD) || !defined(ENABLE_ACLK)
-    netdata_log_info("The claiming feature has been explicitly disabled");
-    *message = strdupz("This agent cannot be claimed, it was built without support for Cloud");
-    return CMD_STATUS_FAILURE;
-#endif
-    netdata_log_info("COMMAND: Reloading Agent Claiming configuration.");
-    claim_reload_all();
+static cmd_status_t cmd_reload_claiming_state_execute(char *args __maybe_unused, char **message) {
+    char msg[1024];
+
+    CLOUD_STATUS status = claim_reload_and_wait_online();
+    switch(status) {
+        case CLOUD_STATUS_ONLINE:
+            snprintfz(msg, sizeof(msg),
+                      "Netdata Agent is claimed to Netdata Cloud and is currently online.");
+            break;
+
+        case CLOUD_STATUS_BANNED:
+            snprintfz(msg, sizeof(msg),
+                      "Netdata Agent is claimed to Netdata Cloud, but it is banned.");
+            break;
+
+        default:
+        case CLOUD_STATUS_AVAILABLE:
+            snprintfz(msg, sizeof(msg),
+                      "Netdata Agent is not claimed to Netdata Cloud: %s",
+                      claim_agent_failure_reason_get());
+            break;
+
+        case CLOUD_STATUS_OFFLINE:
+            snprintfz(msg, sizeof(msg),
+                      "Netdata Agent is claimed to Netdata Cloud, but it is currently offline: %s",
+                      cloud_status_aclk_offline_reason());
+            break;
+
+        case CLOUD_STATUS_INDIRECT:
+            snprintfz(msg, sizeof(msg),
+                      "Netdata Agent is not claimed to Netdata Cloud, but it is currently online via parent.");
+            break;
+    }
+
+    *message = strdupz(msg);
+
     return CMD_STATUS_SUCCESS;
 }
 
@@ -242,9 +252,8 @@ static cmd_status_t cmd_read_config_execute(char *args, char **message)
     const char *conf_file = temp; /* "cloud" is cloud.conf, otherwise netdata.conf */
     struct config *tmp_config = strcmp(conf_file, "cloud") ? &netdata_config : &cloud_config;
 
-    char *value = appconfig_get(tmp_config, temp + offset + 1, temp + offset2 + 1, NULL);
-    if (value == NULL)
-    {
+    const char *value = inicfg_get(tmp_config, temp + offset + 1, temp + offset2 + 1, NULL);
+    if (value == NULL) {
         netdata_log_error("Cannot execute read-config conf_file=%s section=%s / key=%s because no value set",
                           conf_file,
                           temp + offset + 1,
@@ -252,13 +261,11 @@ static cmd_status_t cmd_read_config_execute(char *args, char **message)
         freez(temp);
         return CMD_STATUS_FAILURE;
     }
-    else
-    {
+    else {
         (*message) = strdupz(value);
         freez(temp);
         return CMD_STATUS_SUCCESS;
     }
-
 }
 
 static cmd_status_t cmd_write_config_execute(char *args, char **message)
@@ -287,7 +294,7 @@ static cmd_status_t cmd_write_config_execute(char *args, char **message)
     const char *conf_file = temp; /* "cloud" is cloud.conf, otherwise netdata.conf */
     struct config *tmp_config = strcmp(conf_file, "cloud") ? &netdata_config : &cloud_config;
 
-    appconfig_set(tmp_config, temp + offset + 1, temp + offset2 + 1, temp + offset3 + 1);
+    inicfg_set(tmp_config, temp + offset + 1, temp + offset2 + 1, temp + offset3 + 1);
     netdata_log_info("write-config conf_file=%s section=%s key=%s value=%s",conf_file, temp + offset + 1, temp + offset2 + 1,
          temp + offset3 + 1);
     freez(temp);
@@ -306,17 +313,10 @@ static cmd_status_t cmd_ping_execute(char *args, char **message)
 static cmd_status_t cmd_aclk_state(char *args, char **message)
 {
     netdata_log_info("COMMAND: Reopening aclk/cloud state.");
-#ifdef ENABLE_ACLK
     if (strstr(args, "json"))
         *message = aclk_state_json();
     else
         *message = aclk_state();
-#else
-    if (strstr(args, "json"))
-        *message = strdupz("{\"aclk-available\":false}");
-    else
-        *message = strdupz("ACLK Available: No");
-#endif
 
     return CMD_STATUS_SUCCESS;
 }
@@ -338,49 +338,64 @@ static cmd_status_t cmd_dumpconfig(char *args, char **message)
     (void)args;
 
     BUFFER *wb = buffer_create(1024, NULL);
-    config_generate(wb, 0);
+    inicfg_generate(&netdata_config, wb, 0, true);
     *message = strdupz(buffer_tostring(wb));
     buffer_free(wb);
     return CMD_STATUS_SUCCESS;
 }
 
-#ifdef ENABLE_ACLK
-
-static int remove_ephemeral_host(BUFFER *wb, RRDHOST *host, bool report_error)
+static int remove_ephemeral_host(BUFFER *wb, RRDHOST *host, bool report_error, bool unregister)
 {
     if (host == localhost) {
         if (report_error)
-            buffer_sprintf(wb, "You cannot unregister the parent node (%s)", rrdhost_hostname(host));
+            buffer_sprintf(wb, "Node '%s' (machine guid: %s) is our localhost - not changing it",
+                           rrdhost_hostname(host), host->machine_guid);
         return 0;
     }
 
     if (rrdhost_is_online(host)) {
         if (report_error)
-            buffer_sprintf(wb, "Cannot unregister a live node (%s)", rrdhost_hostname(host));
+            buffer_sprintf(wb, "Node '%s' (machine guid: %s) is online - not changing it",
+                           rrdhost_hostname(host), host->machine_guid);
         return 0;
     }
 
+    bool marked = false;
     if (!rrdhost_option_check(host, RRDHOST_OPTION_EPHEMERAL_HOST)) {
         rrdhost_option_set(host, RRDHOST_OPTION_EPHEMERAL_HOST);
-        sql_set_host_label(&host->host_uuid, "_is_ephemeral", "true");
+        marked = true;
+    }
+
+    sql_set_host_label(&host->host_id.uuid, "_is_ephemeral", "true");
+    pulse_host_status(host, 0, 0);
+
+    if(unregister) {
         aclk_host_state_update(host, 0, 0);
         unregister_node(host->machine_guid);
-        freez(host->node_id);
-        host->node_id = NULL;
-        buffer_sprintf(wb, "Unregistering node with machine guid %s, hostname = %s", host->machine_guid, rrdhost_hostname(host));
+        host->node_id = UUID_ZERO;
+        buffer_sprintf(wb, "Node '%s' (machine guid: %s) has been unregistered",
+                       rrdhost_hostname(host), host->machine_guid);
         rrd_wrlock();
-        rrdhost_free___while_having_rrd_wrlock(host, true);
+        rrdhost_free___while_having_rrd_wrlock(host);
         rrd_wrunlock();
         return 1;
     }
-    if (report_error)
-       buffer_sprintf(wb, "Node with machine guid %s, hostname = %s is already unregistered", host->machine_guid, rrdhost_hostname(host));
+    else if(marked) {
+        buffer_sprintf(wb, "Node '%s' (machine guid: %s) has been marked ephemeral",
+                       rrdhost_hostname(host), host->machine_guid);
+        return 1;
+    }
+    else if (report_error) {
+        buffer_sprintf(wb, "Node '%s' (machine guid: %s) is already ephemeral - not changing it",
+                       rrdhost_hostname(host), host->machine_guid);
+    }
+
     return 0;
 }
 
 #define SQL_HOSTNAME_TO_REMOVE "SELECT host_id FROM host WHERE (hostname = @hostname OR @hostname = 'ALL_NODES')"
 
-static cmd_status_t cmd_remove_node(char *args, char **message)
+static cmd_status_t cmd_remove_stale_node_internal(char *args, char **message, bool unregister)
 {
     (void)args;
 
@@ -393,12 +408,12 @@ static cmd_status_t cmd_remove_node(char *args, char **message)
     RRDHOST *host = NULL;
     host = rrdhost_find_by_guid(args);
     if (!host)
-        host = find_host_by_node_id(args);
+        host = rrdhost_find_by_node_id(args);
 
     if (!host) {
         sqlite3_stmt *res = NULL;
 
-        bool report_error = strcmp(args, "ALL_NODES");
+        bool report_error = strcmp(args, "ALL_NODES") != 0;
 
         if (!PREPARE_STATEMENT(db_meta, SQL_HOSTNAME_TO_REMOVE, &res)) {
             buffer_sprintf(wb, "Failed to prepare database statement to check for stale nodes");
@@ -415,9 +430,11 @@ static cmd_status_t cmd_remove_node(char *args, char **message)
             uuid_unparse_lower(*(nd_uuid_t *)sqlite3_column_blob(res, 0), guid);
             host = rrdhost_find_by_guid(guid);
             if (host) {
-                if (cnt)
+                int rc = remove_ephemeral_host(wb, host, report_error, unregister);
+                if(rc) {
+                    cnt += rc;
                     buffer_fast_strcat(wb, "\n", 1);
-                cnt += remove_ephemeral_host(wb, host, report_error);
+                }
             }
         }
         if (!cnt && buffer_strlen(wb) == 0) {
@@ -431,14 +448,23 @@ static cmd_status_t cmd_remove_node(char *args, char **message)
         SQLITE_FINALIZE(res);
     }
     else
-        (void) remove_ephemeral_host(wb, host, true);
+        (void) remove_ephemeral_host(wb, host, true, unregister);
 
 done:
     *message = strdupz(buffer_tostring(wb));
     buffer_free(wb);
     return CMD_STATUS_SUCCESS;
 }
-#endif
+
+static cmd_status_t cmd_remove_stale_node(char *args, char **message)
+{
+    return cmd_remove_stale_node_internal(args, message, true);
+}
+
+static cmd_status_t cmd_mark_stale_nodes_ephemeral(char *args, char **message)
+{
+    return cmd_remove_stale_node_internal(args, message, false);
+}
 
 static void cmd_lock_exclusive(unsigned index)
 {
@@ -509,7 +535,7 @@ static void pipe_write_cb(uv_write_t* req, int status)
 
 static inline void add_char_to_command_reply(BUFFER *reply_string, unsigned *reply_string_size, char character)
 {
-    buffer_fast_charcat(reply_string, character);
+    buffer_putc(reply_string, character);
     *reply_string_size +=1;
 }
 
