@@ -28,6 +28,7 @@ static struct rrdengine_datafile *datafile_alloc_and_init(struct rrdengine_insta
     datafile->fileno = fileno;
     fatal_assert(0 == uv_rwlock_init(&datafile->extent_rwlock));
     datafile->ctx = ctx;
+    datafile->magic1 = datafile->magic2 = DATAFILE_MAGIC;
 
     datafile->users.available = true;
 
@@ -38,7 +39,7 @@ static struct rrdengine_datafile *datafile_alloc_and_init(struct rrdengine_insta
     return datafile;
 }
 
-bool datafile_acquire(struct rrdengine_datafile *df, DATAFILE_ACQUIRE_REASONS reason) {
+ALWAYS_INLINE bool datafile_acquire(struct rrdengine_datafile *df, DATAFILE_ACQUIRE_REASONS reason) {
     bool ret;
 
     spinlock_lock(&df->users.spinlock);
@@ -94,8 +95,8 @@ bool datafile_acquire_for_deletion(struct rrdengine_datafile *df, bool is_shutdo
             // count the number of pages referencing this in the open cache
             spinlock_unlock(&df->users.spinlock);
             usec_t time_to_scan_ut = now_monotonic_usec();
-            size_t clean_pages_in_open_cache = pgc_count_clean_pages_having_data_ptr(open_cache, (Word_t)df->ctx, df);
-            size_t hot_pages_in_open_cache = pgc_count_hot_pages_having_data_ptr(open_cache, (Word_t)df->ctx, df);
+            size_t clean_pages_in_open_cache = pgc_count_clean_pages_having_data_ptr(open_cache, (Word_t)datafile_ctx(df), df);
+            size_t hot_pages_in_open_cache = pgc_count_hot_pages_having_data_ptr(open_cache, (Word_t)datafile_ctx(df), df);
             time_to_scan_ut = now_monotonic_usec() - time_to_scan_ut;
             spinlock_lock(&df->users.spinlock);
 
@@ -115,7 +116,7 @@ bool datafile_acquire_for_deletion(struct rrdengine_datafile *df, bool is_shutdo
                                          "%zu clean and %zu hot open cache pages "
                                          "- will be deleted shortly "
                                          "(scanned open cache in %"PRIu64" usecs)",
-                                   df->fileno, df->ctx->config.tier,
+                                   df->fileno, datafile_ctx(df)->config.tier,
                                    df->users.lockers,
                                    df->users.lockers_by_reason[DATAFILE_ACQUIRE_OPEN_CACHE],
                                    df->users.lockers_by_reason[DATAFILE_ACQUIRE_PAGE_DETAILS],
@@ -132,7 +133,7 @@ bool datafile_acquire_for_deletion(struct rrdengine_datafile *df, bool is_shutdo
                                          "%zu clean and %zu hot open cache pages "
                                          "- will be deleted now "
                                          "(scanned open cache in %"PRIu64" usecs)",
-                                   df->fileno, df->ctx->config.tier,
+                                   df->fileno, datafile_ctx(df)->config.tier,
                                    df->users.lockers,
                                    df->users.lockers_by_reason[DATAFILE_ACQUIRE_OPEN_CACHE],
                                    df->users.lockers_by_reason[DATAFILE_ACQUIRE_PAGE_DETAILS],
@@ -146,7 +147,7 @@ bool datafile_acquire_for_deletion(struct rrdengine_datafile *df, bool is_shutdo
                                      "has %u lockers (oc:%u, pd:%u), "
                                      "%zu clean and %zu hot open cache pages "
                                      "(scanned open cache in %"PRIu64" usecs)",
-                               df->fileno, df->ctx->config.tier,
+                               df->fileno, datafile_ctx(df)->config.tier,
                                df->users.lockers,
                                df->users.lockers_by_reason[DATAFILE_ACQUIRE_OPEN_CACHE],
                                df->users.lockers_by_reason[DATAFILE_ACQUIRE_PAGE_DETAILS],
@@ -163,12 +164,12 @@ bool datafile_acquire_for_deletion(struct rrdengine_datafile *df, bool is_shutdo
 void generate_datafilepath(struct rrdengine_datafile *datafile, char *str, size_t maxlen)
 {
     (void) snprintfz(str, maxlen - 1, "%s/" DATAFILE_PREFIX RRDENG_FILE_NUMBER_PRINT_TMPL DATAFILE_EXTENSION,
-                    datafile->ctx->config.dbfiles_path, datafile->tier, datafile->fileno);
+                    datafile_ctx(datafile)->config.dbfiles_path, datafile->tier, datafile->fileno);
 }
 
 int close_data_file(struct rrdengine_datafile *datafile)
 {
-    struct rrdengine_instance *ctx = datafile->ctx;
+    struct rrdengine_instance *ctx = datafile_ctx(datafile);
     uv_fs_t req;
     int ret;
     char path[RRDENG_PATH_MAX];
@@ -187,7 +188,7 @@ int close_data_file(struct rrdengine_datafile *datafile)
 
 int unlink_data_file(struct rrdengine_datafile *datafile)
 {
-    struct rrdengine_instance *ctx = datafile->ctx;
+    struct rrdengine_instance *ctx = datafile_ctx(datafile);
     uv_fs_t req;
     int ret;
     char path[RRDENG_PATH_MAX];
@@ -208,7 +209,7 @@ int unlink_data_file(struct rrdengine_datafile *datafile)
 
 int destroy_data_file_unsafe(struct rrdengine_datafile *datafile)
 {
-    struct rrdengine_instance *ctx = datafile->ctx;
+    struct rrdengine_instance *ctx = datafile_ctx(datafile);
     uv_fs_t req;
     int ret;
     char path[RRDENG_PATH_MAX];
@@ -243,7 +244,7 @@ int destroy_data_file_unsafe(struct rrdengine_datafile *datafile)
 
 int create_data_file(struct rrdengine_datafile *datafile)
 {
-    struct rrdengine_instance *ctx = datafile->ctx;
+    struct rrdengine_instance *ctx = datafile_ctx(datafile);
     uv_fs_t req;
     uv_file file;
     int ret, fd;
@@ -260,10 +261,7 @@ int create_data_file(struct rrdengine_datafile *datafile)
     datafile->file = file;
     __atomic_add_fetch(&ctx->stats.datafile_creations, 1, __ATOMIC_RELAXED);
 
-    ret = posix_memalign((void *)&superblock, RRDFILE_ALIGNMENT, sizeof(*superblock));
-    if (unlikely(ret)) {
-        fatal("DBENGINE: posix_memalign:%s", strerror(ret));
-    }
+    (void)posix_memalignz((void *)&superblock, RRDFILE_ALIGNMENT, sizeof(*superblock));
     memset(superblock, 0, sizeof(*superblock));
     (void) strncpy(superblock->magic_number, RRDENG_DF_MAGIC, RRDENG_MAGIC_SZ);
     (void) strncpy(superblock->version, RRDENG_DF_VER, RRDENG_VER_SZ);
@@ -278,7 +276,7 @@ int create_data_file(struct rrdengine_datafile *datafile)
         ctx_io_error(ctx);
     }
     uv_fs_req_cleanup(&req);
-    posix_memfree(superblock);
+    posix_memalign_freez(superblock);
     if (ret < 0) {
         destroy_data_file_unsafe(datafile);
         return ret;
@@ -297,10 +295,7 @@ static int check_data_file_superblock(uv_file file)
     uv_buf_t iov;
     uv_fs_t req;
 
-    ret = posix_memalign((void *)&superblock, RRDFILE_ALIGNMENT, sizeof(*superblock));
-    if (unlikely(ret)) {
-        fatal("DBENGINE: posix_memalign:%s", strerror(ret));
-    }
+    (void)posix_memalignz((void *)&superblock, RRDFILE_ALIGNMENT, sizeof(*superblock));
     iov = uv_buf_init((void *)superblock, sizeof(*superblock));
 
     ret = uv_fs_read(NULL, &req, file, &iov, 1, 0, NULL);
@@ -321,13 +316,13 @@ static int check_data_file_superblock(uv_file file)
         ret = 0;
     }
     error:
-    posix_memfree(superblock);
+        posix_memalign_freez(superblock);
     return ret;
 }
 
 static int load_data_file(struct rrdengine_datafile *datafile)
 {
-    struct rrdengine_instance *ctx = datafile->ctx;
+    struct rrdengine_instance *ctx = datafile_ctx(datafile);
     uv_fs_t req;
     uv_file file;
     int ret, fd, error;
@@ -607,6 +602,9 @@ void finalize_data_files(struct rrdengine_instance *ctx)
         spinlock_unlock(&datafile->writers.spinlock);
         uv_rwlock_wrunlock(&ctx->datafiles.rwlock);
 
+        memset(journalfile, 0, sizeof(*journalfile));
+        memset(datafile, 0, sizeof(*datafile));
+
         freez(journalfile);
         freez(datafile);
 
