@@ -46,9 +46,6 @@ static void main_cache_flush_dirty_page_callback(PGC *cache __maybe_unused, PGC_
         descr->page_length = pgd_disk_footprint(descr->pgd);
 
         DOUBLE_LINKED_LIST_APPEND_ITEM_UNSAFE(base, descr, link.prev, link.next);
-
-        // TODO: ask @stelfrag/@ktsaou about this.
-        // internal_fatal(descr->page_length > RRDENG_BLOCK_SIZE, "DBENGINE: faulty page length calculation");
     }
 
     struct completion completion;
@@ -81,7 +78,7 @@ static void extent_cache_flush_dirty_page_callback(PGC *cache __maybe_unused, PG
     ;
 }
 
-inline TIME_RANGE_COMPARE is_page_in_time_range(time_t page_first_time_s, time_t page_last_time_s, time_t wanted_start_time_s, time_t wanted_end_time_s) {
+ALWAYS_INLINE_HOT TIME_RANGE_COMPARE is_page_in_time_range(time_t page_first_time_s, time_t page_last_time_s, time_t wanted_start_time_s, time_t wanted_end_time_s) {
     // page_first_time_s <= wanted_end_time_s && page_last_time_s >= wanted_start_time_s
 
     if(page_last_time_s < wanted_start_time_s)
@@ -93,7 +90,7 @@ inline TIME_RANGE_COMPARE is_page_in_time_range(time_t page_first_time_s, time_t
     return PAGE_IS_IN_RANGE;
 }
 
-static inline struct page_details *pdc_find_page_for_time(
+static ALWAYS_INLINE_HOT struct page_details *pdc_find_page_for_time(
         Pcvoid_t PArray,
         time_t wanted_time_s,
         size_t *gaps,
@@ -214,7 +211,7 @@ static inline struct page_details *pdc_find_page_for_time(
     return NULL;
 }
 
-static size_t get_page_list_from_pgc(PGC *cache, METRIC *metric, struct rrdengine_instance *ctx,
+static ALWAYS_INLINE_HOT size_t get_page_list_from_pgc(PGC *cache, METRIC *metric, struct rrdengine_instance *ctx,
         time_t wanted_start_time_s, time_t wanted_end_time_s,
         Pvoid_t *JudyL_page_array, size_t *cache_gaps,
         bool open_cache_mode, PDC_PAGE_STATUS tags) {
@@ -357,7 +354,7 @@ static void pgc_inject_gap(struct rrdengine_instance *ctx, METRIC *metric, time_
     pgc_page_release(main_cache, page);
 }
 
-static size_t list_has_time_gaps(
+static ALWAYS_INLINE_HOT size_t list_has_time_gaps(
         struct rrdengine_instance *ctx,
         METRIC *metric,
         Pvoid_t JudyL_page_array,
@@ -491,7 +488,7 @@ static size_t list_has_time_gaps(
 // ----------------------------------------------------------------------------
 
 typedef void (*page_found_callback_t)(PGC_PAGE *page, void *data);
-static size_t get_page_list_from_journal_v2(struct rrdengine_instance *ctx, METRIC *metric, usec_t start_time_ut, usec_t end_time_ut, page_found_callback_t callback, void *callback_data) {
+static ALWAYS_INLINE_HOT size_t get_page_list_from_journal_v2(struct rrdengine_instance *ctx, METRIC *metric, usec_t start_time_ut, usec_t end_time_ut, page_found_callback_t callback, void *callback_data) {
     nd_uuid_t *uuid = mrg_metric_uuid(main_mrg, metric);
     Word_t metric_id = mrg_metric_id(main_mrg, metric);
 
@@ -517,11 +514,20 @@ static size_t get_page_list_from_journal_v2(struct rrdengine_instance *ctx, METR
             continue;
 
         time_t journal_start_time_s = (time_t)(j2_header->start_time_ut / USEC_PER_SEC);
+        size_t journal_v2_file_size = datafile->journalfile->mmap.size;
 
         // the datafile possibly contains useful data for this query
 
         size_t journal_metric_count = (size_t)j2_header->metric_count;
         struct journal_metric_list *uuid_list = (struct journal_metric_list *)((uint8_t *) j2_header + j2_header->metric_offset);
+        size_t metric_offset = (uint8_t *) uuid_list - (uint8_t *) j2_header;
+        if (metric_offset >= journal_v2_file_size) {
+            nd_log_limit_static_thread_var(erl, 60, 0);
+            nd_log_limit(&erl, NDLS_DAEMON, NDLP_ERR, "DBENGINE: Invalid metric list header in journalfile %u of tier %u", datafile->fileno, datafile->tier);
+            journalfile_v2_data_release(datafile->journalfile);
+            continue;
+        }
+
         struct journal_metric_list *uuid_entry = bsearch(uuid,uuid_list,journal_metric_count,sizeof(*uuid_list), journal_metric_uuid_compare);
 
         if (unlikely(!uuid_entry)) {
@@ -531,8 +537,17 @@ static size_t get_page_list_from_journal_v2(struct rrdengine_instance *ctx, METR
         }
 
         struct journal_page_header *page_list_header = (struct journal_page_header *) ((uint8_t *) j2_header + uuid_entry->page_offset);
+        size_t page_offset = (uint8_t *) page_list_header - (uint8_t *) j2_header;
+        if (page_offset >= journal_v2_file_size) {
+            nd_log_limit_static_thread_var(erl, 60, 0);
+            nd_log_limit(&erl, NDLS_DAEMON, NDLP_ERR, "DBENGINE: Invalid page list header in journalfile %u of tier %u", datafile->fileno, datafile->tier);
+            journalfile_v2_data_release(datafile->journalfile);
+            continue;
+        }
+
         struct journal_page_list *page_list = (struct journal_page_list *)((uint8_t *) page_list_header + sizeof(*page_list_header));
         struct journal_extent_list *extent_list = (void *)((uint8_t *)j2_header + j2_header->extent_offset);
+        uint32_t extent_entries = j2_header->extent_count;
         uint32_t uuid_page_entries = page_list_header->entries;
 
         for (uint32_t index = 0; index < uuid_page_entries; index++) {
@@ -548,6 +563,13 @@ static size_t get_page_list_from_journal_v2(struct rrdengine_instance *ctx, METR
             if(prc == PAGE_IS_IN_THE_FUTURE)
                 break;
 
+            // Make sure index is valid for this file
+            if (page_entry_in_journal->extent_index > extent_entries) {
+                nd_log_limit_static_thread_var(erl, 60, 0);
+                nd_log_limit(&erl, NDLS_DAEMON, NDLP_ERR, "DBENGINE: Invalid extent index in journalfile %u", datafile->fileno);
+                break;
+            }
+
             uint32_t page_update_every_s = page_entry_in_journal->update_every_s;
             size_t page_length = page_entry_in_journal->page_length;
 
@@ -629,7 +651,7 @@ void add_page_details_from_journal_v2(PGC_PAGE *page, void *JudyL_pptr) {
 // Pvalue of the judy will be the end time for that page
 // DBENGINE2:
 #define time_delta(finish, pass) do { if(pass) { usec_t t = pass; (pass) = (finish) - (pass); (finish) = t; } } while(0)
-static Pvoid_t get_page_list(
+static ALWAYS_INLINE_HOT Pvoid_t get_page_list(
         struct rrdengine_instance *ctx,
         METRIC *metric,
         usec_t start_time_ut,
@@ -756,7 +778,7 @@ we_are_done:
     return JudyL_page_array;
 }
 
-inline void rrdeng_prep_wait(PDC *pdc) {
+ALWAYS_INLINE void rrdeng_prep_wait(PDC *pdc) {
     if (unlikely(pdc && !pdc->prep_done)) {
         usec_t started_ut = now_monotonic_usec();
         completion_wait_for(&pdc->prep_completion);
@@ -765,7 +787,7 @@ inline void rrdeng_prep_wait(PDC *pdc) {
     }
 }
 
-void rrdeng_prep_query(struct page_details_control *pdc, bool worker) {
+ALWAYS_INLINE_HOT void rrdeng_prep_query(struct page_details_control *pdc, bool worker) {
     if(worker)
         worker_is_busy(UV_EVENT_DBENGINE_QUERY);
 
@@ -820,7 +842,7 @@ void rrdeng_prep_query(struct page_details_control *pdc, bool worker) {
  * @param end_time_ut inclusive ending time in usec
  * @return 1 / 0 (pages found or not found)
  */
-void pg_cache_preload(struct rrdeng_query_handle *handle) {
+ALWAYS_INLINE_HOT void pg_cache_preload(struct rrdeng_query_handle *handle) {
     if (unlikely(!handle || !handle->metric))
         return;
 
@@ -1045,31 +1067,31 @@ void pgc_open_add_hot_page(Word_t section, Word_t metric_id, time_t start_time_s
     pgc_page_release(open_cache, (PGC_PAGE *)page);
 }
 
-size_t dynamic_open_cache_size(void) {
-    size_t main_wanted_cache_size = pgc_get_wanted_cache_size(main_cache);
-    size_t target_size = main_wanted_cache_size / 100 * 5;
+int64_t dynamic_open_cache_size(void) {
+    int64_t main_wanted_cache_size = pgc_get_wanted_cache_size(main_cache);
+    int64_t target_size = main_wanted_cache_size / 100 * 5;
 
     if(target_size < 2 * 1024 * 1024)
         target_size = 2 * 1024 * 1024;
 
-    size_t main_current_cache_size = pgc_get_current_cache_size(main_cache);
+    int64_t main_current_cache_size = pgc_get_current_cache_size(main_cache);
 
-    size_t main_free_cache_size = (main_wanted_cache_size > main_current_cache_size) ?
+    int64_t main_free_cache_size = (main_wanted_cache_size > main_current_cache_size) ?
                                       main_wanted_cache_size - main_current_cache_size : 0;
 
     return target_size + main_free_cache_size;
 }
 
-size_t dynamic_extent_cache_size(void) {
-    size_t main_wanted_cache_size = pgc_get_wanted_cache_size(main_cache);
-    size_t target_size = main_wanted_cache_size / 100 * 30;
+int64_t dynamic_extent_cache_size(void) {
+    int64_t main_wanted_cache_size = pgc_get_wanted_cache_size(main_cache);
+    int64_t target_size = main_wanted_cache_size / 100 * 30;
 
     if(target_size < 5 * 1024 * 1024)
         target_size = 5 * 1024 * 1024;
 
-    size_t main_current_cache_size = pgc_get_current_cache_size(main_cache);
+    int64_t main_current_cache_size = pgc_get_current_cache_size(main_cache);
 
-    size_t main_free_cache_size = (main_wanted_cache_size > main_current_cache_size) ?
+    int64_t main_free_cache_size = (main_wanted_cache_size > main_current_cache_size) ?
                                       main_wanted_cache_size - main_current_cache_size : 0;
 
     return target_size + main_free_cache_size;
