@@ -6,7 +6,7 @@
 #include "rrdengine.h"
 
 #define RRDENG_MIN_PAGE_CACHE_SIZE_MB (8)
-#define RRDENG_MIN_DISK_SPACE_MB (256)
+#define RRDENG_MIN_DISK_SPACE_MB (25)
 #define RRDENG_DEFAULT_TIER_DISK_SPACE_MB (1024)
 
 #define RRDENG_NR_STATS (38)
@@ -74,10 +74,11 @@ int rrdeng_init(
 void rrdeng_readiness_wait(struct rrdengine_instance *ctx);
 
 int rrdeng_exit(struct rrdengine_instance *ctx);
-void rrdeng_quiesce(struct rrdengine_instance *ctx);
+void rrdeng_quiesce(struct rrdengine_instance *ctx, bool dirty_only);
 
 bool rrdeng_metric_retention_by_id(STORAGE_INSTANCE *si, UUIDMAP_ID id, time_t *first_entry_s, time_t *last_entry_s);
 bool rrdeng_metric_retention_by_uuid(STORAGE_INSTANCE *si, nd_uuid_t *dim_uuid, time_t *first_entry_s, time_t *last_entry_s);
+void rrdeng_metric_retention_delete_by_id(STORAGE_INSTANCE *si, UUIDMAP_ID id);
 
 extern STORAGE_METRICS_GROUP *rrdeng_metrics_group_get(STORAGE_INSTANCE *si, nd_uuid_t *uuid);
 extern void rrdeng_metrics_group_release(STORAGE_INSTANCE *si, STORAGE_METRICS_GROUP *smg);
@@ -136,7 +137,7 @@ struct time_and_count {
     usec_t usec;
 };
 
-static inline void time_and_count_add(struct time_and_count *tc, usec_t dt) {
+static ALWAYS_INLINE void time_and_count_add(struct time_and_count *tc, usec_t dt) {
     __atomic_add_fetch(&tc->count, 1, __ATOMIC_RELAXED);
     __atomic_add_fetch(&tc->usec, dt, __ATOMIC_RELAXED);
 }
