@@ -183,8 +183,7 @@ static void delete_label(RRDLABEL *label)
 // ----------------------------------------------------------------------------
 // rrdlabels_destroy()
 
-void rrdlabels_destroy(RRDLABELS *labels)
-{
+void rrdlabels_flush(RRDLABELS *labels) {
     if (unlikely(!labels))
         return;
 
@@ -199,6 +198,14 @@ void rrdlabels_destroy(RRDLABELS *labels)
     size_t memory_freed = JudyLFreeArray(&labels->JudyL, PJE0);
     STATS_MINUS_MEMORY(&dictionary_stats_category_rrdlabels, 0, memory_freed + sizeof(RRDLABELS), 0);
     spinlock_unlock(&labels->spinlock);
+}
+
+void rrdlabels_destroy(RRDLABELS *labels)
+{
+    if (unlikely(!labels))
+        return;
+
+    rrdlabels_flush(labels);
     freez(labels);
 }
 
@@ -395,7 +402,14 @@ void rrdlabels_key_to_buffer_array_item(RRDLABELS *labels, BUFFER *wb)
 // ----------------------------------------------------------------------------
 
 void rrdlabels_get_value_strcpyz(RRDLABELS *labels, char *dst, size_t dst_len, const char *key) {
-    if(!labels) return;
+    if(!dst || !dst_len)
+        return;
+
+    // make sure the output is empty if we can't find it
+    *dst = '\0';
+
+    if(!labels)
+        return;
 
     STRING *this_key = string_strdupz(key);
 
@@ -406,7 +420,7 @@ void rrdlabels_get_value_strcpyz(RRDLABELS *labels, char *dst, size_t dst_len, c
     {
         if (lb->index.key == this_key) {
             if (lb->index.value)
-                strncpyz(dst, string2str(lb->index.value), dst_len);
+                strncpyz(dst, string2str(lb->index.value), dst_len - 1);
             else
                 dst[0] = '\0';
             break;
