@@ -12,8 +12,10 @@
 #define WORKER_HEALTH_JOB_CRITICAL_EVAL         5
 #define WORKER_HEALTH_JOB_ALARM_LOG_ENTRY       6
 #define WORKER_HEALTH_JOB_ALARM_LOG_PROCESS     7
-#define WORKER_HEALTH_JOB_DELAYED_INIT_RRDSET   8
-#define WORKER_HEALTH_JOB_DELAYED_INIT_RRDDIM   9
+#define WORKER_HEALTH_JOB_ALARM_LOG_QUEUE       8
+#define WORKER_HEALTH_JOB_WAIT_EXEC             9
+#define WORKER_HEALTH_JOB_DELAYED_INIT_RRDSET   10
+#define WORKER_HEALTH_JOB_DELAYED_INIT_RRDDIM   11
 
 #if WORKER_UTILIZATION_MAX_JOB_TYPES < 10
 #error WORKER_UTILIZATION_MAX_JOB_TYPES has to be at least 10
@@ -148,8 +150,6 @@ static void health_initialize_rrdhost(RRDHOST *host) {
         !service_running(SERVICE_HEALTH))
         return;
 
-    rrdhost_flag_set(host, RRDHOST_FLAG_INITIALIZED_HEALTH);
-
     host->health_log.max = health_globals.config.health_log_entries_max;
     host->health_log.health_log_retention_s = health_globals.config.health_log_retention_s;
     host->health.default_exec = string_dup(health_globals.config.default_exec);
@@ -159,8 +159,10 @@ static void health_initialize_rrdhost(RRDHOST *host) {
     host->health_log.next_log_id = get_uint32_id();
     host->health_log.next_alarm_id = 0;
 
-    rw_spinlock_init(&host->health_log.spinlock);
     sql_health_alarm_log_load(host);
+    rw_spinlock_init(&host->health_log.spinlock);
+    rrdhost_flag_set(host, RRDHOST_FLAG_INITIALIZED_HEALTH);
+
     health_apply_prototypes_to_host(host);
 }
 
@@ -637,10 +639,12 @@ static void health_event_loop_for_host(RRDHOST *host, bool apply_hibernation_del
             wc->send_snapshot = 2;
             rrdhost_flag_set(host, RRDHOST_FLAG_ACLK_STREAM_ALERTS);
         } else {
+            worker_is_busy(WORKER_HEALTH_JOB_ALARM_LOG_QUEUE);
             if (process_alert_pending_queue(host))
                 rrdhost_flag_set(host, RRDHOST_FLAG_ACLK_STREAM_ALERTS);
         }
     }
+    worker_is_idle();
 }
 
 __thread bool is_health_thread = false;
@@ -665,7 +669,7 @@ static void health_event_loop(void) {
                    "Postponing alarm checks for %"PRId32" seconds, "
                    "because it seems that the system was just resumed from suspension.",
                    (int32_t)health_globals.config.postpone_alarms_during_hibernation_for_seconds);
-            schedule_node_state_update(localhost, 0);
+            schedule_node_state_update(localhost, 10);
         }
 
         if (unlikely(silencers->all_alarms && silencers->stype == STYPE_DISABLE_ALARMS)) {
@@ -693,7 +697,9 @@ static void health_event_loop(void) {
             break;
 
         // wait for all notifications to finish before allowing health to be cleaned up
+        worker_is_busy(WORKER_HEALTH_JOB_WAIT_EXEC);
         wait_for_all_notifications_to_finish_before_allowing_health_to_be_cleaned_up();
+        worker_is_idle();
         
         health_sleep(next_run, loop);
     } // forever
@@ -721,6 +727,8 @@ void *health_main(void *ptr) {
     worker_register_job_name(WORKER_HEALTH_JOB_CRITICAL_EVAL, "critical eval");
     worker_register_job_name(WORKER_HEALTH_JOB_ALARM_LOG_ENTRY, "alert log entry");
     worker_register_job_name(WORKER_HEALTH_JOB_ALARM_LOG_PROCESS, "alert log process");
+    worker_register_job_name(WORKER_HEALTH_JOB_ALARM_LOG_QUEUE, "alert log queue");
+    worker_register_job_name(WORKER_HEALTH_JOB_WAIT_EXEC, "alert wait exec");
     worker_register_job_name(WORKER_HEALTH_JOB_DELAYED_INIT_RRDSET, "rrdset init");
     worker_register_job_name(WORKER_HEALTH_JOB_DELAYED_INIT_RRDDIM, "rrddim init");
 
