@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 
-#include "daemon/common.h"
+#include "database/rrd.h"
 #include "KolmogorovSmirnovDist.h"
 
 #define MAX_POINTS 10000
